"""
To run comment in runNoCallBack for 300ms (about 90 seconds)
or runCallBack 2000ms (200 minutes).  

This actually does the full thing:  it views from the retina, through V1,
to pyramid detection, then sets a fact on.  It also has the planning in.
So, it does the commands that are activated.  This can be set in the
short version in runNoCallBack, or all can be tested with the RunCallBack
version.
"""

#imports for PyNN
from pyNN.utility import get_script_args, normalized_filename
from neo.io import PyNNTextIO

#imports for image reading
import numpy
from PIL import Image
#Other NEAL files and functions
from retinaConnections import set3x3OnConnections,set3x3OffConnections
from retinaConnections import set6x6OnConnections, set6x6OffConnections
from retinaConnections import set9x9OnConnections,set9x9OffConnections
from v1Connections import set6x6OnToHedgeBottomConnections, set6x6OffToHedgeBottomConnections
from v1Connections import set6x6OnToSedgeRightConnections
from v1Connections import set6x6OnToBedgeLeftConnections
from v1Connections import set3x3OnToAndAngleConnections,set9x9OffToAndAngleConnections
from v1Connections import set3x3OnToLessThanAngleConnections,set9x9OffToLessThanAngleConnections
from v1Connections import set3x3OnToGreaterThanAngleConnections,set9x9OffToGreaterThanAngleConnections

INPUT_NEURONS_HEIGHT = 50
INPUT_NEURONS_WIDTH = 50

DELAY = 1.0
output_filename='cabot1_out.dat'
SIM_LENGTH = 300.0
#SIM_LENGTH = 2000.0 #For the full plan test

CA_HEIGHT = 10
CA_WIDTH = 10

#----------------setup connections-----------------
#setup CAs in the pyramid net
def connectCA (CANum):
    CASize = CA_WIDTH*CA_HEIGHT
    CARow = CANum/5
    CACol = CANum%5
    synWeight = 3.0
    numSynapses = 10
    connector = []
    for cRow in range (0,CA_HEIGHT):
        for cCol in range (0,CA_WIDTH):
            fromNeuron=cRow*INPUT_NEURONS_WIDTH+cCol
            fromNeuron=fromNeuron+(CARow*CA_HEIGHT*INPUT_NEURONS_WIDTH)
            fromNeuron = fromNeuron + (CACol*CA_WIDTH)
            for cSynapse in range (0,numSynapses):
                toCol = cCol + cSynapse + 1
                toRow = cRow
                if (toCol>=CA_WIDTH) :
                    toCol = toCol - 10
                    toRow = toRow + 1
                if (toRow>=CA_HEIGHT) :
                    toRow = toRow - 10
                toNeuron=toRow*INPUT_NEURONS_WIDTH+toCol
                toNeuron=toNeuron+(CARow*CA_HEIGHT*INPUT_NEURONS_WIDTH)
                toNeuron = toNeuron + (CACol*CA_WIDTH)
                connector = connector + [(fromNeuron,toNeuron,synWeight,DELAY)]

    return connector

#add connections up 5 and over one both ways (15)
def connectHEdgeBottomToPyramid ():
    totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
    synWeight = 10.0
    connector = []
    for cNeuron in range (INPUT_NEURONS_WIDTH*10,totalNeurons):
       fromRow = cNeuron/50
       fromCol = cNeuron % 50
       for toRow in range(fromRow-5,fromRow):
          for toCol in range (fromCol-1,fromCol + 2):
             toNeuron = toRow*50+toCol
             connector = connector + [(cNeuron,toNeuron,synWeight,DELAY)]
    return connector

#add connections down left (7:30) 5 and over one both ways (15)
def connectSEdgeRightToPyramid ():
    totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
    synWeight = 10.0
    connector = []
    for cNeuron in range (5,totalNeurons-(INPUT_NEURONS_WIDTH*10)):
       fromRow = cNeuron/50
       fromCol = cNeuron % 50
       if fromCol < 5:
          continue
       for rowChange in range(1,6):
          toRow = fromRow+rowChange
          for toCol in range (fromCol-rowChange-1,fromCol-rowChange + 2):
             toNeuron = toRow*50+toCol
             connector = connector + [(cNeuron,toNeuron,synWeight,DELAY)]
    return connector
 
#bedge
#add connections down right (4:30) 5 and over one both ways (15)
def connectBEdgeLeftToPyramid ():
    totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
    synWeight = 10.0
    connector = []
    for cNeuron in range (0,totalNeurons-(INPUT_NEURONS_WIDTH*10)):
       fromRow = cNeuron/50
       fromCol = cNeuron % 50
       if fromCol > 45:
          continue
       for rowChange in range(1,6):
          toRow = fromRow+rowChange
          for toCol in range (fromCol+rowChange-1,fromCol+rowChange + 2):
             toNeuron = toRow*50+toCol
             connector = connector + [(cNeuron,toNeuron,synWeight,DELAY)]
    return connector

#add connections down 5 and over one both ways (15)
def connectAndAngleToPyramid ():
    totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
    synWeight = 10.0
    connector = []
    for cNeuron in range (0,totalNeurons-(INPUT_NEURONS_WIDTH*10)):
       fromRow = cNeuron/50
       fromCol = cNeuron % 50
       for toRow in range(fromRow+1,fromRow+6):
          for toCol in range (fromCol-1,fromCol + 2):
             toNeuron = toRow*50+toCol
             connector = connector + [(cNeuron,toNeuron,synWeight,DELAY)]
    return connector
       
#add connections up right (1:30) 5 and over one both ways (15)
def connectLessThanAngleToPyramid ():
    totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
    synWeight = 10.0
    connector = []
    for cNeuron in range (INPUT_NEURONS_WIDTH*5,totalNeurons):
       fromRow = cNeuron/50
       fromCol = cNeuron % 50
       if fromCol > 45:
          continue
       for rowChange in range(-5,0):
          toRow = fromRow+rowChange
          for toCol in range (fromCol-rowChange-1,fromCol-rowChange + 2):
             toNeuron = toRow*50+toCol
             connector = connector + [(cNeuron,toNeuron,synWeight,DELAY)]
    return connector
   
#add connections up left (half ten) 5 and over one both ways (15)
def connectGreaterThanAngleToPyramid ():
    totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
    synWeight = 10.0
    connector = []
    for cNeuron in range (INPUT_NEURONS_WIDTH*5,totalNeurons):
       fromRow = cNeuron/50
       fromCol = cNeuron % 50
       if fromCol < 6:
          continue
       for rowChange in range(-5,0):
          toRow = fromRow+rowChange
          for toCol in range (fromCol+rowChange-1,fromCol+rowChange + 2):
             toNeuron = toRow*50+toCol
             connector = connector + [(cNeuron,toNeuron,synWeight,DELAY)]
    return connector
   

#------set up object to fact connections----------
#-every neuron of the left 20 of the pyramid turns on the left flag
def leftPyramidTurnsLeftFactOn ():
    synWeight = 3.0
    connector = []
    for cRow in range (0,INPUT_NEURONS_HEIGHT):
        for cCol in range (0,int(INPUT_NEURONS_WIDTH*0.4)):
            fromNeuron=cRow*INPUT_NEURONS_WIDTH+cCol
            toNeuron = cCol % 5
            connector = connector + [(fromNeuron,toNeuron,synWeight,DELAY)]
    return connector
    


#----------------functions for setting plan connections
def setStandardOscillatorConnector(oscPart1,oscPart2,weight):
	connect(oscPart1,oscPart2,weight, DELAY)

def setPlanConnections():
    weight = 6.0
    synWeight = 6.0
    inhibitW = -5.25
    synWeightCoeff = 0.25
    #connections for facts
    setStandardOscillatorConnector(fact_pyr_right_1,fact_pyr_right_2,weight)
    setStandardOscillatorConnector(fact_pyr_right_2,fact_pyr_right_1,weight)

    #connections for goals
    setStandardOscillatorConnector(goalTL_1,goalTL_2,weight)
    setStandardOscillatorConnector(goalTL_2,goalTL_1,weight)
    setStandardOscillatorConnector(goalTR_1,goalTR_2,weight)
    setStandardOscillatorConnector(goalTR_2,goalTR_1,weight)
    setStandardOscillatorConnector(goalBack_1,goalBack_2,weight)
    setStandardOscillatorConnector(goalBack_2,goalBack_1,weight)
    setStandardOscillatorConnector(goalForward_1,goalForward_2,weight)
    setStandardOscillatorConnector(goalForward_2,goalForward_1,weight)
    setStandardOscillatorConnector(goal_move_left1_1,goal_move_left1_2,weight)
    setStandardOscillatorConnector(goal_move_left1_2,goal_move_left1_1,weight)
    setStandardOscillatorConnector(goal_move_left2_1,goal_move_left2_2,weight)
    setStandardOscillatorConnector(goal_move_left2_2,goal_move_left2_1,weight)
    setStandardOscillatorConnector(goal_move_right1_1,goal_move_right1_2,weight)
    setStandardOscillatorConnector(goal_move_right1_2,goal_move_right1_1,weight)
    setStandardOscillatorConnector(goal_move_right2_1,goal_move_right2_2,weight)
    setStandardOscillatorConnector(goal_move_right2_2,goal_move_right2_1,weight)
    setStandardOscillatorConnector(goal_pyramid_1,goal_pyramid_2,weight)
    setStandardOscillatorConnector(goal_pyramid_2,goal_pyramid_1,weight)


    #connect compound goals to second part of goals
    setStandardOscillatorConnector(goal_move_left1_1,goal_move_left2_1,synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_move_left1_2,goal_move_left2_1,synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_move_right1_1,goal_move_right2_1,synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_move_right1_2,goal_move_right2_1,synWeightCoeff*synWeight)

    #connections for modules
    setStandardOscillatorConnector(moduleTL_1,moduleTL_2,weight)
    setStandardOscillatorConnector(moduleTL_2,moduleTL_1,weight)
    setStandardOscillatorConnector(moduleTR_1,moduleTR_2,weight)
    setStandardOscillatorConnector(moduleTR_2,moduleTR_1,weight)
    setStandardOscillatorConnector(moduleBack_1,moduleBack_2,weight)
    setStandardOscillatorConnector(moduleBack_2,moduleBack_1,weight)
    setStandardOscillatorConnector(moduleForward_1,moduleForward_2,weight)
    setStandardOscillatorConnector(moduleForward_2,moduleForward_1,weight)

    #connections between modules and goals
    setStandardOscillatorConnector(goalTL_1,moduleTL_2,weight)
    setStandardOscillatorConnector(goalTL_2,moduleTL_1,weight)
    setStandardOscillatorConnector(goalTL_1,moduleTL_1,weight)
    setStandardOscillatorConnector(goalTL_2,moduleTL_2,weight)
    setStandardOscillatorConnector(goalTR_1,moduleTR_2,weight)
    setStandardOscillatorConnector(goalTR_2,moduleTR_1,weight)
    setStandardOscillatorConnector(goalTR_1,moduleTR_1,weight)
    setStandardOscillatorConnector(goalTR_2,moduleTR_2,weight)
    setStandardOscillatorConnector(goalBack_1,moduleBack_2,weight)
    setStandardOscillatorConnector(goalBack_2,moduleBack_1,weight)
    setStandardOscillatorConnector(goalBack_1,moduleBack_1,weight)
    setStandardOscillatorConnector(goalBack_2,moduleBack_2,weight)
    setStandardOscillatorConnector(goalForward_1,moduleForward_2,weight)
    setStandardOscillatorConnector(goalForward_2,moduleForward_1,weight)
    setStandardOscillatorConnector(goalForward_1,moduleForward_1,weight)
    setStandardOscillatorConnector(goalForward_2,moduleForward_2,weight)
    setStandardOscillatorConnector(goal_move_left1_1,moduleTL_2,weight)
    setStandardOscillatorConnector(goal_move_left1_2,moduleTL_1,weight)
    setStandardOscillatorConnector(goal_move_left1_1,moduleTL_1,weight)
    setStandardOscillatorConnector(goal_move_left1_2,moduleTL_2,weight)
    setStandardOscillatorConnector(goal_move_left2_1,moduleForward_2,weight)
    setStandardOscillatorConnector(goal_move_left2_2,moduleForward_1,weight)
    setStandardOscillatorConnector(goal_move_left2_1,moduleForward_1,weight)
    setStandardOscillatorConnector(goal_move_left2_2,moduleForward_2,weight)
    setStandardOscillatorConnector(goal_move_right1_1,moduleTR_2,weight)
    setStandardOscillatorConnector(goal_move_right1_2,moduleTR_1,weight)
    setStandardOscillatorConnector(goal_move_right1_1,moduleTR_1,weight)
    setStandardOscillatorConnector(goal_move_right1_2,moduleTR_2,weight)
    setStandardOscillatorConnector(goal_move_right2_1,moduleForward_2,weight)
    setStandardOscillatorConnector(goal_move_right2_2,moduleForward_1,weight)
    setStandardOscillatorConnector(goal_move_right2_1,moduleForward_1,weight)
    setStandardOscillatorConnector(goal_move_right2_2,moduleForward_2,weight)

    #connections between goal_pyramid (find the pyramid) and modules.
    #very weak and passive, since pyramid could be either left or right
    setStandardOscillatorConnector(goal_pyramid_1, moduleTR_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_1, moduleTR_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_2, moduleTR_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_2, moduleTR_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_1, moduleTL_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_1, moduleTL_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_2, moduleTL_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(goal_pyramid_2,moduleTL_2,synWeightCoeff*synWeight)

    #facts have stronger weights and more bias, excite the **_1  lead to spike
    synWeightCoeff = 0.75
    chrisWeight = 1.0
    setStandardOscillatorConnector(leftFactAPop,moduleTL_1,chrisWeight)
    setStandardOscillatorConnector(leftFactBPop,moduleTL_1,chrisWeight)
    setStandardOscillatorConnector(fact_pyr_right_1,moduleTR_1,synWeightCoeff*synWeight)
    setStandardOscillatorConnector(fact_pyr_right_2,moduleTR_1,synWeightCoeff*synWeight)

    #facts inhibit modules
    setStandardOscillatorConnector(fact_pyr_right_1,moduleTL_1,inhibitW)
    setStandardOscillatorConnector(fact_pyr_right_1,moduleTL_2,inhibitW)
    chrisInhib = -3.0
    setStandardOscillatorConnector(leftFactAPop,moduleTR_1,chrisInhib)
    setStandardOscillatorConnector(leftFactAPop,moduleTR_2,chrisInhib)

    #to and from actions
    setStandardOscillatorConnector(moduleTL_1, TL, synWeight)
    setStandardOscillatorConnector(moduleTL_2, TL, synWeight)
    setStandardOscillatorConnector(TL,moduleTL_1, inhibitW)
    setStandardOscillatorConnector(TL,moduleTL_2, inhibitW)
    setStandardOscillatorConnector(TL,goalTL_1, inhibitW)
    setStandardOscillatorConnector(TL,goalTL_2, inhibitW)

    setStandardOscillatorConnector(moduleTR_1, TR, synWeight)
    setStandardOscillatorConnector(moduleTR_2, TR, synWeight)
    setStandardOscillatorConnector(TR,moduleTR_1, inhibitW)
    setStandardOscillatorConnector(TR,moduleTR_2, inhibitW)
    setStandardOscillatorConnector(TR,goalTR_1, inhibitW)
    setStandardOscillatorConnector(TR,goalTR_2, inhibitW)

    setStandardOscillatorConnector(moduleBack_1, back, synWeight)
    setStandardOscillatorConnector(moduleBack_2, back, synWeight)
    setStandardOscillatorConnector(back,moduleBack_1, inhibitW)
    setStandardOscillatorConnector(back,moduleBack_2, inhibitW)
    setStandardOscillatorConnector(back,goalBack_1, inhibitW)
    setStandardOscillatorConnector(back,goalBack_2, inhibitW)

    setStandardOscillatorConnector(moduleForward_1, forward, synWeight)
    setStandardOscillatorConnector(moduleForward_2, forward, synWeight)
    setStandardOscillatorConnector(forward,moduleForward_1, inhibitW)
    setStandardOscillatorConnector(forward,moduleForward_2, inhibitW)
    setStandardOscillatorConnector(forward,goalForward_1, inhibitW)
    setStandardOscillatorConnector(forward,goalForward_2, inhibitW)

    #action left and forward for move left
    synWeightCoeff=0.75
    setStandardOscillatorConnector(TL, goal_move_left2_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(TL, goal_move_left2_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(TL, goal_move_left1_1, inhibitW)
    setStandardOscillatorConnector(TL, goal_move_left1_2, inhibitW)
    setStandardOscillatorConnector(forward, goal_move_left2_1, inhibitW)
    setStandardOscillatorConnector(forward, goal_move_left2_2, inhibitW)

    setStandardOscillatorConnector(TR, goal_move_right2_1, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(TR, goal_move_right2_2, synWeightCoeff*synWeight)
    setStandardOscillatorConnector(TR, goal_move_right1_1, inhibitW)
    setStandardOscillatorConnector(TR, goal_move_right1_2, inhibitW)
    setStandardOscillatorConnector(forward, goal_move_right2_1, inhibitW)
    setStandardOscillatorConnector(forward, goal_move_right2_2, inhibitW)

    #actions turn off facts too (thanks Ian)
    setStandardOscillatorConnector(TR, fact_pyr_right_1, inhibitW)
    setStandardOscillatorConnector(TR, fact_pyr_right_2, inhibitW)
    setStandardOscillatorConnector(TL, leftFactAPop, inhibitW)
    setStandardOscillatorConnector(TL, leftFactBPop, inhibitW)

    #actions turn off pyramid goals sJA
    setStandardOscillatorConnector(TL, goal_pyramid_1, inhibitW)
    setStandardOscillatorConnector(TL, goal_pyramid_2, inhibitW)
    #eja
    setStandardOscillatorConnector(TR, goal_pyramid_1, inhibitW)
    setStandardOscillatorConnector(TR, goal_pyramid_2, inhibitW)

#-------------Print out neuron by neuron the firing times
def print_spikes(of, label, pop):
	print "\n\n%s\n" % label
	of.write("\n%s\n" % label)
	outDat = pop.get_data()
	for seg in outDat.segments:
		print(seg.spiketrains)
		of.write("\n%s\n" % str(seg.spiketrains))

def printReceptorFirings(receptorPop):
   totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
   for neuronNum in range (0,totalNeurons-1):
       if (neuronNum < totalNeurons):
           outAssembly = Assembly(receptorPop[neuronNum,(neuronNum+1)])
       else: 
           outAssembly = Assembly(receptorPop[neuronNum,0])
       outDat = outAssembly.get_data()
       seg = outDat.segments[0]
       spikeTrains = seg.spiketrains
       times = spikeTrains[0].times
       if len(times) > 0:
           print neuronNum, len(times),
           for time in times:
               base = time.base
               print base,
           print " "

def printReceptors():
    print "3x3 On"
    printReceptorFirings(retina3x3OnPop)
    print "3x3 Off"
#    printReceptorFirings(retina3x3OffPop)
    print "6x6 On"
    printReceptorFirings(retina6x6OnPop)
    print "6x6 Off"
    printReceptorFirings(retina6x6OffPop)
    print "9x9 On"
#    printReceptorFirings(retina9x9OnPop)
    print "9x9 Off"
    printReceptorFirings(retina9x9OffPop)

def printV1():
    print "HEdgeBottom"
    printReceptorFirings(v1HEdgeBottomPop)
    print "SEdgeRight"
    printReceptorFirings(v1SEdgeRightPop)
    print "BEdgeLeft"
    printReceptorFirings(v1BEdgeLeftPop)
    print "AndAngle"
    printReceptorFirings(v1AndAnglePop)
    print "LessThanAngle"
    printReceptorFirings(v1LessThanAnglePop)
    print "GreaterThanAngle"
    printReceptorFirings(v1GreaterThanAnglePop)

#------Image Stuff
def openImageFile():
    try: 
#pyramid        im = Image.open("ed_3screenshot.jpg")
#stalactite
        im = Image.open("screenshot538.jpg")
    except:
        print "fail to open image"

    im = im.crop((0,0,600,600))
    #print im.size

    #shrink it
    size = (INPUT_NEURONS_WIDTH,INPUT_NEURONS_HEIGHT)
    im.thumbnail(size)

    #get the pixels
    pixels = im.load()

    ##convert rgb to grey scale
    inputMatrix = numpy.zeros((50,50))
    for inpRow in range (0,50):
        for inpCol in range (0,50):
            rgb = pixels[inpCol,inpRow]
            for i in range (0,3):
                inputMatrix[inpCol,inpRow] += rgb[i]
                inputMatrix[inpCol,inpRow]/=3
        
    ##set result image array
    resultImage = numpy.zeros((50,50))
    THRESHOLD = 50
    resultImageArray = []
    for outRow in range (0,50):
        for outColumn in range (0,50):
            if inputMatrix[outColumn,outRow] < THRESHOLD :
                #print outRow, outColumn
                pixelOn = (outRow*INPUT_NEURONS_WIDTH)+outColumn
                resultImageArray = resultImageArray + [pixelOn]

    return resultImageArray

def injection(start_time, population):
	interval=10.0
	pulse=DCSource(amplitude=0.0055, start=start_time, stop=start_time+interval)
	pulse.inject_into(Assembly(population[0,1,2,3,4]))
		
#stimulus
def report_time(t):
        print t
	if (t==0):
		pulse =DCSource(amplitude=0.0055,start=0.0,stop=10.0)
		TR_Assembly = Assembly(goalTR_1[0,1,2,3,4])
		pulse.inject_into(TR_Assembly)
	elif (t == 100):
		pulse =DCSource(amplitude=0.0055,start=100.0,stop=110.0)
		TL_Assembly = Assembly(goalTL_1[0,1,2,3,4])
		pulse.inject_into(TL_Assembly)
	elif (t == 200):
		pulse =DCSource(amplitude=0.0055,start=200.0,stop=210.0)
		forward_Assembly = Assembly(goalForward_1[0,1,2,3,4])
		pulse.inject_into(forward_Assembly)
	elif (t == 300):
		pulse =DCSource(amplitude=0.0055,start=300.0,stop=310.0)
		back_Assembly = Assembly(goalBack_1[0,1,2,3,4])
		pulse.inject_into(back_Assembly)
	elif (t == 400):
		pulse =DCSource(amplitude=0.0055,start=400.0,stop=410.0)
		move_left1_Assembly = Assembly(goal_move_left1_1[0,1,2,3,4])
		#move_left2_Assembly = Assembly(goal_move_left2_1[0,1])
		pulse.inject_into(move_left1_Assembly)
		#pulse.inject_into(move_left2_Assembly)
	elif (t == 500):
		pulse = DCSource(amplitude=0.0055, start=500.0, stop=510.0)
		move_right1_Assembly = Assembly(goal_move_right1_1[0,1,2,3,4])
		#move_right2_Assembly = Assembly(goal_move_right2_1[0,1])
		pulse.inject_into(move_right1_Assembly)
		#pulse.inject_into(move_right2_Assembly)
	elif (t==600):
		pulse = DCSource(amplitude=0.0055, start=600.0, stop=610.0)
		move_left1_Assembly = Assembly(goal_move_left1_1[0,1,2,3,4])
		pulse.inject_into(move_left1_Assembly)
	elif (t==700):
		injection(700.0, goalTR_1)
	elif (t==800):
		injection(800.0, goalTL_1)
	elif (t==900):
		pulse =DCSource(amplitude=0.0055,start=900.0,stop=910.0)
		forward_Assembly = Assembly(goalForward_1[0,1,2,3,4])
		pulse.inject_into(forward_Assembly)
	elif (t==1000):
		pulse =DCSource(amplitude=0.0055, start=1000.0,stop=1010.0)
		TR_Assembly = Assembly(goalTR_1[0,1,2,3,4])
		pulse.inject_into(TR_Assembly)
	elif (t==1100):
		pulse =DCSource(amplitude=0.0055,start=1100.0,stop=1110.0)
		TL_Assembly = Assembly(goalTL_1[0,1,2,3,4])
		pulse.inject_into(TL_Assembly)
	elif (t==1200):
		injection(1200, goal_pyramid_1)
                #undone this is cheating
		#injection(1200, fact_pyr_left_1)
	elif(t==1300):
		injection(1300, goal_pyramid_1)
                #undone this is cheating but needs to be tested
		#injection(1300, fact_pyr_right_1)
	elif(t==1400):
		injection(t,goalTR_1)
	elif(t==1500):
		injection(t,goalTL_1)
	return t + DELAY


def runNoCallBack():
    #set up clamped input from image
    pulseVis = DCSource(amplitude=0.0038, start=0.0, stop=SIM_LENGTH)
    inputAssembly = Assembly(inpPop[imagePixels])
    pulseVis.inject_into(inputAssembly)

    pulseGoal = DCSource(amplitude=0.0038, start=0.0, stop=10)
    #goalInputAssembly = Assembly(goal_move_left1_1[0,1,2,3,4])
    goalInputAssembly = Assembly(goal_pyramid_1[0,1,2,3,4])
    pulseGoal.inject_into(goalInputAssembly)

    run(SIM_LENGTH)

def runCallBack():
    #set up clamped input from image
    pulseVis = DCSource(amplitude=0.0038, start=0.0, stop=SIM_LENGTH)
    inputAssembly = Assembly(inpPop[imagePixels])
    pulseVis.inject_into(inputAssembly)

    run(SIM_LENGTH, callbacks=[report_time])

#------------Main Body---------------
simulator_name = get_script_args(1)[0]  
exec("from pyNN.%s import *" % simulator_name)

setup(timestep=DELAY,min_delay=DELAY,max_delay=DELAY,debug=0)

#----------------create neurons
#input neurons
cInputNeurons = INPUT_NEURONS_HEIGHT * INPUT_NEURONS_WIDTH 

#default a .02, b .2, c -65, d 2.0, i_offset 0
inp_cell_params = {'a' : 0.0, 'b' :0.2, 'c' : -65, 'd' : 0.0}
inpPop = Population(cInputNeurons,Izhikevich(**inp_cell_params))

inpPop.initialize(v=-58.0)

#create retinal neurons off centre different than on centre
retina_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0}
retina3x3OnPop  = Population(cInputNeurons,Izhikevich(**retina_cell_params))
retina6x6OnPop  = Population(cInputNeurons,Izhikevich(**retina_cell_params))
retina9x9OnPop  = Population(cInputNeurons,Izhikevich(**retina_cell_params))

retina_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0,'i_offset':1.0}
retina3x3OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))
retina6x6OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))
retina9x9OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))

#create V1 neurons
V1_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0}
v1HEdgeBottomPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1SEdgeRightPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1BEdgeLeftPop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1AndAnglePop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1LessThanAnglePop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))
v1GreaterThanAnglePop  = Population(cInputNeurons,Izhikevich(**V1_cell_params))

#create object recognition neurons
obj_cell_params = {'a' : 0.02,'i_offset' : 2}
pyramidPop = Population(cInputNeurons,Izhikevich(**obj_cell_params))

#create fact neurons (for vision)
fact_cell_params = {'a': 0.05, 'd': 0,'i_offset' : 2}
leftFactAPop = Population(5,Izhikevich(**fact_cell_params))
leftFactBPop = Population(5,Izhikevich(**fact_cell_params))
    

##create plan neurons
cell_params = {'a' : 0.05}
goalTL_1 = Population(5,Izhikevich(**cell_params))
goalTL_2 = Population(5,Izhikevich(**cell_params))

goalTR_1 = Population(5, Izhikevich(**cell_params))
goalTR_2 = Population(5, Izhikevich(**cell_params))

goalBack_1 = Population(5,Izhikevich(**cell_params))
goalBack_2 = Population(5,Izhikevich(**cell_params))

goalForward_1 = Population(5, Izhikevich(**cell_params))
goalForward_2 = Population(5, Izhikevich(**cell_params))

goal_move_left1_1 = Population(5, Izhikevich(**cell_params))
goal_move_left1_2 = Population(5, Izhikevich(**cell_params))

goal_move_left2_1 = Population(5, Izhikevich(**cell_params))
goal_move_left2_2 = Population(5, Izhikevich(**cell_params))

goal_move_right1_1 = Population(5, Izhikevich(**cell_params))
goal_move_right1_2 = Population(5, Izhikevich(**cell_params))

goal_move_right2_1 = Population(5, Izhikevich(**cell_params))
goal_move_right2_2 = Population(5, Izhikevich(**cell_params))


goal_pyramid_1 = Population(5, Izhikevich(**cell_params))
goal_pyramid_2 = Population(5, Izhikevich(**cell_params))


#FACTS	
fact_pyr_right_1 = Population(5, Izhikevich(**cell_params))
fact_pyr_right_2 = Population(5, Izhikevich(**cell_params))


#modules 
moduleTL_1 = Population(5,Izhikevich(**cell_params))
moduleTL_2 = Population(5,Izhikevich(**cell_params))
moduleTR_1 = Population(5, Izhikevich(**cell_params))
moduleTR_2 = Population(5, Izhikevich(**cell_params))
moduleBack_1 = Population(5,Izhikevich(**cell_params))
moduleBack_2 = Population(5,Izhikevich(**cell_params))
moduleForward_1 = Population(5, Izhikevich(**cell_params))
moduleForward_2 = Population(5, Izhikevich(**cell_params))


#output/actions
TL = Population(1, Izhikevich(**cell_params))
TR = Population(1, Izhikevich(**cell_params))
back = Population(1, Izhikevich(**cell_params))
forward = Population(1, Izhikevich(**cell_params))


#----------------setup input to retina connections
synapseArray = []
#3x3On
for row in range (1,49):
    for col in range (1,49):
        newSynapses = set3x3OnConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(inpPop,retina3x3OnPop,FromListConnector(synapseArray),StaticSynapse())

#3x3Off
#synapseArray = []
#for row in range (1,49):
#    for col in range (1,49):
#        newSynapses = set3x3OffConnections(col,row)
#        synapseArray = synapseArray + newSynapses
#Projection(inpPop,retina3x3OffPop,FromListConnector(synapseArray),StaticSynapse())

#6x6On
synapseArray = []
for row in range (2,47):
    for col in range (2,47):
        newSynapses = set6x6OnConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(inpPop,retina6x6OnPop,FromListConnector(synapseArray),StaticSynapse())

#6x6Off
synapseArray = []
for row in range (2,47):
    for col in range (2,47):
        newSynapses = set6x6OffConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(inpPop,retina6x6OffPop,FromListConnector(synapseArray),StaticSynapse())

#9x9Off
synapseArray = []
for row in range (3,45):
    for col in range (3,45):
        newSynapses = set9x9OffConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(inpPop,retina9x9OffPop,FromListConnector(synapseArray),StaticSynapse())

#9x9On
#synapseArray = []
#for row in range (3,45):
#    for col in range (3,45):
#        newSynapses = set9x9OnConnections(col,row)
#        synapseArray = synapseArray + newSynapses
#Projection(inpPop,retina9x9OnPop,FromListConnector(synapseArray),StaticSynapse())

#-------------------setup retina to V1 connections---
#Hedge
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (2,INPUT_NEURONS_HEIGHT-3):
        newSynapses = set6x6OnToHedgeBottomConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina6x6OnPop,v1HEdgeBottomPop,FromListConnector(synapseArray),StaticSynapse())
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT-1):
    for col in range (2,INPUT_NEURONS_HEIGHT-3):
        newSynapses = set6x6OffToHedgeBottomConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina6x6OffPop,v1HEdgeBottomPop,FromListConnector(synapseArray),StaticSynapse())

#SEdge Bottom Connections
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set6x6OnToSedgeRightConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina6x6OnPop,v1SEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())
Projection(retina6x6OffPop,v1SEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())

#leftBEdges
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set6x6OnToBedgeLeftConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina6x6OnPop,v1BEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())
Projection(retina6x6OffPop,v1BEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())

#andAngle
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set3x3OnToAndAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina3x3OnPop,v1AndAnglePop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set9x9OffToAndAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina9x9OffPop,v1AndAnglePop,FromListConnector(synapseArray),StaticSynapse())

#Less Than Angle
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set3x3OnToLessThanAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina3x3OnPop,v1LessThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set9x9OffToLessThanAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses
Projection(retina9x9OffPop,v1LessThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

#Greater Than Angle
synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set3x3OnToGreaterThanAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina3x3OnPop,v1GreaterThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

synapseArray = []
for row in range (0,INPUT_NEURONS_HEIGHT):
    for col in range (0,INPUT_NEURONS_WIDTH):
        newSynapses = set9x9OffToGreaterThanAngleConnections(col,row)
        synapseArray = synapseArray + newSynapses

Projection(retina9x9OffPop,v1GreaterThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

#-------------------setup Object Recognion and  V1 to ObjRec connections---
print "V1 connections"
#setup pyramid CAs
synapseArray = []
for cCA in range (0,25):
    newSynapses = connectCA(cCA)
    synapseArray = synapseArray + newSynapses
Projection(pyramidPop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#HEdge to Pyramid
synapseArray = connectHEdgeBottomToPyramid()
Projection(v1HEdgeBottomPop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#SEdge to Pyramid
synapseArray = connectSEdgeRightToPyramid()
Projection(v1SEdgeRightPop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#BEdge to Pyramid
synapseArray = connectBEdgeLeftToPyramid()
Projection(v1BEdgeLeftPop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())

#And to Pyramid
synapseArray = connectAndAngleToPyramid()
Projection(v1AndAnglePop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())


#Less Than Angle to Pyramid
synapseArray = connectLessThanAngleToPyramid()
Projection(v1LessThanAnglePop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())


#Greater Than Angle to Pyramid
synapseArray = connectGreaterThanAngleToPyramid()
Projection(v1GreaterThanAnglePop,pyramidPop,FromListConnector(synapseArray),StaticSynapse())


#setup fact connections
synWeight = 5.0
connE = connect(leftFactAPop, leftFactBPop, weight=synWeight, delay=DELAY)
connE = connect(leftFactBPop, leftFactAPop, weight=synWeight, delay=DELAY)

#connect objects to facts
synapseArray = leftPyramidTurnsLeftFactOn()
Projection(pyramidPop,leftFactAPop,FromListConnector(synapseArray),StaticSynapse())

print "Plan Connections"
setPlanConnections()

#-------------------setup recording
inpPop.record(['spikes','v'])
retina3x3OnPop.record(['spikes','v'])
#retina3x3OffPop.record(['spikes','v'])
retina6x6OnPop.record(['spikes','v'])
retina6x6OffPop.record(['spikes','v'])
#retina9x9OnPop.record(['spikes','v'])
retina9x9OffPop.record(['spikes','v'])
v1HEdgeBottomPop.record(['spikes','v'])
v1SEdgeRightPop.record(['spikes','v'])
v1BEdgeLeftPop.record(['spikes','v'])
v1AndAnglePop.record(['spikes','v'])
v1LessThanAnglePop.record(['spikes','v'])
v1GreaterThanAnglePop.record(['spikes','v'])
pyramidPop.record(['spikes','v'])
leftFactAPop.record(['spikes','v'])
back.record('spikes')
forward.record('spikes')
TR.record('spikes')
TL.record('spikes')

#comment - open file
of = open(output_filename, 'w')

#print inpPop[0].get_parameters()

#open image file
imagePixels = openImageFile()
#print imagePixels

runNoCallBack()
#runCallBack()

#--------------print results
outAss = Assembly(leftFactAPop[0,1])
#outAss = Assembly(pyramidPop[1010,1269])
#outAss =  v1HEdgeBottomPop
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)
#    print(seg.analogsignalarrays)
#printReceptors()
#printV1()

print_spikes(of,"Pop Back", back)
print_spikes(of,"Pop Forward", forward)
print_spikes(of,"Pop Turn Right", TR)
print_spikes(of,"Pop Turn Left", TL)



end()

